/*
 * This software is governed by the CeCILL-B license under French law and
 * abiding by the rules of distribution of free software. You can use, modify
 * and/ or redistribute the software under the terms of the CeCILL-B license as
 * circulated by CEA, CNRS and INRIA at the following URL "http://www.cecill.info".
 * 
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated with
 * loading, using, modifying and/or developing or reproducing the software by
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate, and that also therefore means that it is
 * reserved for developers and experienced professionals having in-depth
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling
 * the security of their systems and/or data to be ensured and, more generally,
 * to use and operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */

package fr.amap.commons.util.colors;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.LinearGradientPaint;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.util.HashMap;
import java.util.Map;
import javax.swing.ImageIcon;

/**
 * Class to create and store colorbars.
 */
public class ColorBars {

    private static final Map<String, Color[]> COLOR_BARS = new HashMap();

    private static void initialize() {
        COLOR_BARS.put("Minnesota", createGradient(
                new Color(0xA0, 0x00, 0x00), new Color(0xFF, 0xFF, 0x00), 500)
        );

        COLOR_BARS.put("Temperature", createGradient(
                Color.BLUE, Color.RED, 500)
        );

        COLOR_BARS.put("Mono", createGradient(
                Color.BLACK, Color.WHITE, 500)
        );

        COLOR_BARS.put("Ice Cream", createGradient(
                Color.WHITE, Color.RED, 500)
        );

        COLOR_BARS.put("Amazonia", createGradient(
                Color.RED, Color.GREEN, 500)
        );

        COLOR_BARS.put("Equatorial", createMultiGradient(
                500, Color.green, Color.yellow, Color.orange, Color.red)
        );
        
        COLOR_BARS.put("Rainbow", createMultiGradient(
                500, Color.blue, Color.cyan, Color.green, 
                Color.yellow, Color.orange, Color.red, Color.pink)
        );
        
        COLOR_BARS.put("Hot", createMultiGradient(
                500, Color.black, new Color(87, 0, 0), Color.red,
                Color.orange, Color.yellow, Color.white)
        );
        
        COLOR_BARS.put("Heat", createMultiGradient(
                500, Color.black, new Color(105, 0, 0),
                new Color(192, 23, 0), new Color(255, 150, 38), Color.white)
        );
        
        COLOR_BARS.put("Roy", createMultiGradient(
                500, Color.red, Color.orange, Color.yellow)
        );
        
        COLOR_BARS.put("Three", createMultiGradient(
                500, new Color(125, 129, 0), new Color(90, 164, 0), new Color(173, 81, 0))
        );
    }

    public static final Color[] getColorBar(final String name) {
        if (COLOR_BARS.isEmpty()) {
            initialize();
        }

        return COLOR_BARS.get(name);
    }
    
    public static final int getIndexOf(final String name) {
        int index = 0;
        
        for (String colorBarsName : getColorBarsNames()) {
            if (colorBarsName == null ? name == null : colorBarsName.equals(name)) {
                return index;
            }
            
            index++;
        }
        
        return index;
    }

    public static final Map<String, Color[]> getColorBars() {
        return COLOR_BARS;
    }

    public static final String[] getColorBarsNames() {
        if (COLOR_BARS.isEmpty()) {
            initialize();
        }

        return COLOR_BARS.keySet().toArray(new String[COLOR_BARS.size()]);
    }

    /**
     * Creates an array of Color objects for use as a gradient, using a linear
     * interpolation between the two specified colors.
     * @param one Color used for the bottom of the gradient
     * @param two Color used for the top of the gradient
     * @param numSteps The number of steps in the gradient. 250 is a good
     * number.
     * @return
     */
    private static Color[] createGradient(Color one, Color two, int numSteps) {
        int r1 = one.getRed();
        int g1 = one.getGreen();
        int b1 = one.getBlue();

        int r2 = two.getRed();
        int g2 = two.getGreen();
        int b2 = two.getBlue();

        int newR;
        int newG;
        int newB;

        Color[] gradient = new Color[numSteps];
        double iNorm;
        for (int i = 0; i < numSteps; i++) {
            iNorm = i / (double) numSteps; // a normalized [0:1] variable
            newR = (int) (r1 + iNorm * (r2 - r1));
            newG = (int) (g1 + iNorm * (g2 - g1));
            newB = (int) (b1 + iNorm * (b2 - b1));
            gradient[i] = new Color(newR, newG, newB);
        }

        return gradient;
    }

    /**
     * Creates an array of Color objects for use as a gradient, using an array
     * of Color objects. It uses a linear interpolation between each pair of
     * points.
     * @param colors array of Color objects used for the gradient. The Color at
     * index 0 will be the lowest color.
     * @param numSteps The number of steps in the gradient. 250 is a good
     * number.
     * @return
     */
    private static Color[] createMultiGradient(int numSteps, Color... colors) {
        // we assume a linear gradient, with equal spacing between colors
        // The final gradient will be made up of n 'sections', where n =
        // colors.length - 1
        int numSections = colors.length - 1;
        int gradientIndex = 0; // points to the next open spot in the final
        // gradient
        Color[] gradient = new Color[numSteps];
        Color[] temp;

        if (numSections <= 0) {
            throw new IllegalArgumentException(
                    "You must pass in at least 2 colors in the array!");
        }

        for (int section = 0; section < numSections; section++) {
            // we divide the gradient into (n - 1) sections, and do a regular
            // gradient for each
            temp = createGradient(colors[section], colors[section + 1],
                    numSteps / numSections);
            for (Color temp1 : temp) {
                // copy the sub-gradient into the overall gradient
                gradient[gradientIndex++] = temp1;
            }
        }

        if (gradientIndex < numSteps) {
            // The rounding didn't work out in our favor, and there is at least
            // one unfilled slot in the gradient[] array.
            // We can just copy the final color there
            for (/* nothing to initialize */; gradientIndex < numSteps; gradientIndex++) {
                gradient[gradientIndex] = colors[colors.length - 1];
            }
        }

        return gradient;
    }
    
    public static ImageIcon asIcon(Color[] colorbar, int width, int height) {
        if (colorbar == null || width <= 0 || height <= 0) {
            return null;
        }
        
        /* Creat image */
        final BufferedImage img = new BufferedImage(width, height, 
                BufferedImage.TYPE_INT_RGB);
        final Graphics2D g = (Graphics2D) img.getGraphics();
        
        /* Draw a white rectangle */
        final Rectangle2D r = new Rectangle2D.Double(0, 0, width, height);
        g.setColor(new Color(0.95f, 0.95f, 0.95f));
        g.fill(r);
        
        final float circleSize = height * 0.825f;
        final float rectangleSize = height * 0.85f;
        final float borderSize = 1f;
        
        /* Draw borders */
        g.setColor(Color.GRAY);
        g.fill(new Ellipse2D.Float(width * 0.001f - borderSize, 
                (height - circleSize) / 2 - borderSize, 
                circleSize + borderSize * 2, circleSize + borderSize * 1.25f));
        
        g.setColor(Color.GRAY);
        g.fill(new Ellipse2D.Float(width - circleSize - width * 0.005f - borderSize, 
                (height - circleSize) / 2 - borderSize,
                circleSize + borderSize * 2, circleSize + borderSize * 1.25f));
        
        g.setPaint(Color.GRAY);
        g.fill(new Rectangle2D.Float(width * 0.001f + circleSize / 2 - borderSize,
                (height - rectangleSize) * 0.75f - borderSize, 
                width * 0.998f - circleSize + borderSize * 2, rectangleSize + borderSize * 1.25f));
        
        /* Draw a circle at the left */
        g.setColor(colorbar[0]);
        g.fill(new Ellipse2D.Float(width * 0.001f, 
                (height - circleSize) / 2, circleSize, circleSize));
        
        /* Draw a circle at the right */
        g.setColor(colorbar[colorbar.length - 1]);
        g.fill(new Ellipse2D.Float(width - circleSize - width * 0.005f, 
                (height - circleSize) / 2, circleSize, circleSize));

        /* Draw the gradient */
        final float[] fractions = new float[colorbar.length];
        for (int i = 0; i < fractions.length; i++) {
            fractions[i] = 1f / (float) fractions.length * (float) i;
        }
        
        final LinearGradientPaint gradientPaint = new LinearGradientPaint(0, 0,
                width, 0, fractions, colorbar);
        g.setPaint(gradientPaint);
        g.fill(new Rectangle2D.Float(width * 0.001f + circleSize / 2,
                (height - rectangleSize) * 0.75f, 
                width * 0.998f - circleSize, rectangleSize));
        
        return new ImageIcon(img);
    }

}
